@@ -28,73 +28,6 @@
 
 #include "svgint.h"
 
-static xmlEntity *
-_svg_parser_sax_get_entity (void		*closure,
-			    const xmlChar	*name);
-
-static void
-_svg_parser_sax_entity_decl (void		*closure,
-			     const xmlChar	*name,
-			     int		type,
-			     const xmlChar	*publicId,
-			     const xmlChar	*systemId,
-			     xmlChar		*content);
-
-
-static void
-_svg_parser_sax_start_element (void		*closure,
-			       const xmlChar	*name,
-			       const xmlChar	**atts);
-
-static void
-_svg_parser_sax_end_element (void		*closure,
-			     const xmlChar	*name);
-
-static void
-_svg_parser_sax_characters (void		*closure,
-			    const xmlChar	*ch,
-			    int			len);
-
-static void
-_svg_parser_sax_warning (void		*closure,
-			 const char	*msg, ...);
-
-static void
-_svg_parser_sax_error (void		*closure,
-		       const char	*msg, ...);
-
-static void
-_svg_parser_sax_fatal_error (void	*closure,
-			     const char	*msg, ...);
-
-static xmlSAXHandler SVG_PARSER_SAX_HANDLER = {
-    NULL,				/* internalSubset */
-    NULL,				/* isStandalone */
-    NULL,				/* hasInternalSubset */
-    NULL,				/* hasExternalSubset */
-    NULL,				/* resolveEntity */
-    _svg_parser_sax_get_entity,		/* getEntity */
-    _svg_parser_sax_entity_decl,	/* entityDecl */
-    NULL,				/* notationDecl */
-    NULL,				/* attributeDecl */
-    NULL,				/* elementDecl */
-    NULL,				/* unparsedEntityDecl */
-    NULL,				/* setDocumentLocator */
-    NULL,				/* startDocument */
-    NULL,				/* endDocument */
-    _svg_parser_sax_start_element,	/* startElement */
-    _svg_parser_sax_end_element,	/* endElement */
-    NULL,				/* reference */
-    _svg_parser_sax_characters,		/* characters */
-    _svg_parser_sax_characters,		/* ignorableWhitespace */
-    NULL,				/* processingInstruction */
-    NULL,				/* comment */
-    _svg_parser_sax_warning,		/* xmlParserWarning */
-    _svg_parser_sax_error,		/* xmlParserError */
-    _svg_parser_sax_fatal_error,	/* xmlParserFatalError */
-    NULL,				/* getParameterEntity */
-};
-
 static svg_status_t
 _svg_parser_push_state (svg_parser_t		*parser,
 			const svg_parser_cb_t	*cb);
@@ -117,102 +50,102 @@ _svg_parser_new_leaf_element (svg_parser_t *parser,
 
 static svg_status_t
 _svg_parser_parse_anchor (svg_parser_t	*parser,
-			  const xmlChar	**attributes,
+			  const char	**attributes,
 			  svg_element_t	**group_element);
 
 static svg_status_t
 _svg_parser_parse_svg (svg_parser_t	*parser,
-		       const xmlChar	**attributes,
+		       const char	**attributes,
 		       svg_element_t	**group_element);
 
 static svg_status_t
 _svg_parser_parse_defs (svg_parser_t	*parser,
-		       const xmlChar	**attributes,
+		       const char	**attributes,
 		       svg_element_t	**group_element);
 
 static svg_status_t
 _svg_parser_parse_use (svg_parser_t	*parser,
-		       const xmlChar	**attributes,
+		       const char	**attributes,
 		       svg_element_t	**group_element);
 
 static svg_status_t
 _svg_parser_parse_symbol (svg_parser_t	*parser,
-			  const xmlChar	**attributes,
+			  const char	**attributes,
 			  svg_element_t	**group_element);
 
 static svg_status_t
 _svg_parser_parse_group (svg_parser_t	*parser,
-			 const xmlChar	**attributes,
+			 const char	**attributes,
 			 svg_element_t	**group_element);
 
 static svg_status_t
 _svg_parser_parse_path (svg_parser_t	*parser,
-			const xmlChar	**attributes,
+			const char	**attributes,
 			svg_element_t	**path_element);
 
 static svg_status_t
 _svg_parser_parse_line (svg_parser_t	*parser,
-			const xmlChar	**attributes,
+			const char	**attributes,
 			svg_element_t	**path_element);
 
 static svg_status_t
 _svg_parser_parse_rect (svg_parser_t	*parser,
-			const xmlChar	**attributes,
+			const char	**attributes,
 			svg_element_t	**path_element);
 
 static svg_status_t
 _svg_parser_parse_circle (svg_parser_t	*parser,
-			  const xmlChar	**attributes,
+			  const char	**attributes,
 			  svg_element_t	**path_element);
 
 static svg_status_t
 _svg_parser_parse_ellipse (svg_parser_t		*parser,
-			   const xmlChar	**attributes,
+			   const char	**attributes,
 			   svg_element_t	**path_element);
 
 static svg_status_t
 _svg_parser_parse_polygon (svg_parser_t		*parser,
-			   const xmlChar	**attributes,
+			   const char	**attributes,
 			   svg_element_t	**path_element);
 
 static svg_status_t
 _svg_parser_parse_polyline (svg_parser_t	*parser,
-			    const xmlChar	**attributes,
+			    const char	**attributes,
 			    svg_element_t	**path_element);
 
 static svg_status_t
 _svg_parser_parse_text (svg_parser_t	*parser,
-			const xmlChar	**attributes,
+			const char	**attributes,
 			svg_element_t	**text_element);
 
 static svg_status_t
 _svg_parser_parse_image (svg_parser_t	*parser,
-			 const xmlChar	**attributes,
+			 const char	**attributes,
 			 svg_element_t	**image_element);
 
 static svg_status_t
 _svg_parser_parse_linear_gradient (svg_parser_t	*parser,
-				   const xmlChar	**attributes,
+				   const char	**attributes,
 				   svg_element_t	**path_element);
 
 static svg_status_t
 _svg_parser_parse_radial_gradient (svg_parser_t	*parser,
-				   const xmlChar	**attributes,
+				   const char	**attributes,
 				   svg_element_t	**path_element);
 
 static svg_status_t
 _svg_parser_parse_gradient_stop (svg_parser_t	*parser,
-				 const xmlChar	**attributes,
+				 const char	**attributes,
 				 svg_element_t	**stop_element);
 
 static svg_status_t
 _svg_parser_parse_pattern (svg_parser_t	*parser,
-			   const xmlChar	**attributes,
+			   const char	**attributes,
 			   svg_element_t	**path_element);
 
 static svg_status_t
 _svg_parser_parse_text_characters (svg_parser_t		*parser,
-				   const xmlChar	*ch,
+				   const char	*ch,
 				   int			len);
 
 typedef struct svg_parser_map {
@@ -243,148 +176,17 @@ static const svg_parser_map_t SVG_PARSER_MAP[] = {
     {"pattern",		{_svg_parser_parse_pattern,		NULL }},
 };
 
-svg_status_t
-_svg_parser_init (svg_parser_t *parser, svg_t *svg)
-{
-    parser->svg = svg;
-    parser->ctxt = NULL;
-
-    parser->unknown_element_depth = 0;
-
-    parser->state = NULL;
-
-    parser->status = SVG_STATUS_SUCCESS;
-
-    return parser->status;
-}
-
-svg_status_t
-_svg_parser_deinit (svg_parser_t *parser)
-{
-    parser->svg = NULL;
-    parser->ctxt = NULL;
-
-    parser->status = SVG_STATUS_SUCCESS;
-
-    return parser->status;
-}
-
-svg_status_t
-_svg_parser_begin (svg_parser_t *parser)
-{
-    /* Innocent until proven guilty */
-    parser->status = SVG_STATUS_SUCCESS;
-
-    if (parser->ctxt)
-	parser->status = SVG_STATUS_INVALID_CALL;
-
-    parser->ctxt = xmlCreatePushParserCtxt (&SVG_PARSER_SAX_HANDLER,
-					    parser, NULL, 0, NULL);
-    if (parser->ctxt == NULL)
-	parser->status = SVG_STATUS_NO_MEMORY;
-
-    parser->ctxt->replaceEntities = 1;
-
-    parser->entities = xmlHashCreate (100);
-    return parser->status;
-}
-
-svg_status_t
-_svg_parser_parse_chunk (svg_parser_t *parser, const char *buf, size_t count)
-{
-    if (parser->status)
-	return parser->status;
-
-    if (parser->ctxt == NULL)
-	return SVG_STATUS_INVALID_CALL;
-
-    xmlParseChunk (parser->ctxt, buf, count, 0);
-
-    return parser->status;
-}
-
-svg_status_t
-_svg_parser_end (svg_parser_t *parser)
-{
-    if (parser->ctxt == NULL)
-	return SVG_STATUS_INVALID_CALL;
-
-    if (! parser->ctxt->wellFormed)
-	parser->status = SVG_STATUS_PARSE_ERROR;
-
-    xmlFreeParserCtxt (parser->ctxt);
-    parser->ctxt = NULL;
-
-    xmlHashFree (parser->entities, (xmlHashDeallocator) xmlFree);
-    parser->entities = NULL;
-
-    return parser->status;
-}
-
-static xmlEntity *
-_svg_parser_sax_get_entity (void		*closure,
-			    const xmlChar	*name)
-{
-    svg_parser_t *parser = closure;
-
-    return xmlHashLookup (parser->entities, name);
-}
-
-/* XXX: It's not clear to my why we have to do this. libxml2 has all
- * of this code internally, so we should be able to access a hash of
- * entities automatically rather than maintaining our own. */
-static void
-_svg_parser_sax_entity_decl (void		*closure,
-			     const xmlChar	*name,
-			     int		type,
-			     const xmlChar	*publicId,
-			     const xmlChar	*systemId,
-			     xmlChar		*content)
-{
-    svg_parser_t *parser = closure;
-    xmlEntityPtr entity;
-
-    entity = malloc (sizeof(xmlEntity));
-
-    entity->type = XML_ENTITY_DECL;
-    entity->name = xmlMemStrdup (name);
-    entity->etype = type;
-    if (publicId)
-	entity->ExternalID = xmlMemStrdup (publicId);
-    else
-	entity->ExternalID = NULL;
-    if (systemId)
-	entity->SystemID = xmlMemStrdup (systemId);
-    else
-	entity->SystemID = NULL;
-
-    if (content) {
-	entity->length = xmlStrlen (content);
-	entity->content = xmlStrndup (content, entity->length);
-    } else {
-	entity->length = 0;
-	entity->content = NULL;
-    }
-    entity->URI = NULL;
-    entity->orig = NULL;
-    entity->owner = 0;
-    entity->children = NULL;
-
-    if (xmlHashAddEntry (parser->entities, name, entity)) {
-	/* Entity was already defined at another level. */
-	free (entity);
-    }
-}
-
-static void
+void
 _svg_parser_sax_start_element (void		*closure,
-			       const xmlChar	*name,
-			       const xmlChar	**attributes)
+			       const xmlChar	*name_unsigned,
+			       const xmlChar	**attributes_unsigned)
 {
     unsigned int i;
     svg_parser_t *parser = closure;
     const svg_parser_cb_t *cb;
     svg_element_t *element;
+    const char *name = (const char *) name_unsigned;
+    const char **attributes = (const char **) attributes_unsigned;
 
     if (parser->unknown_element_depth) {
 	parser->unknown_element_depth++;
@@ -425,7 +227,7 @@ _svg_parser_sax_start_element (void		*closure,
     return;
 }
 
-static void
+void
 _svg_parser_sax_end_element (void		*closure,
 			     const xmlChar	*name)
 {
@@ -443,15 +245,15 @@ _svg_parser_sax_end_element (void		*closure,
     return;
 }
 
-static void
+void
 _svg_parser_sax_characters (void		*closure,
-			    const xmlChar	*ch,
+			    const xmlChar	*ch_unsigned,
 			    int			len)
 {
     int i;
     svg_parser_t *parser = closure;
-    const xmlChar *src;
-    xmlChar *ch_copy, *dst;
+    const char *src, *ch = (const char *) ch_unsigned;
+    char *ch_copy, *dst;
     int space;
 
     /* XXX: This is the correct default behavior, but we're supposed
@@ -491,36 +293,6 @@ _svg_parser_sax_characters (void		*closure,
     return;
 }
 
-static void
-_svg_parser_sax_warning (void *closure, const char *msg, ...)
-{
-    va_list args;
-	
-    va_start (args, msg);
-    vfprintf (stderr, msg, args);
-    va_end (args);
-}
-
-static void
-_svg_parser_sax_error (void *closure, const char *msg, ...)
-{
-    va_list args;
-	
-    va_start (args, msg);
-    vfprintf (stderr, msg, args);
-    va_end (args);
-}
-
-static void
-_svg_parser_sax_fatal_error (void *closure, const char *msg, ...)
-{
-    va_list args;
-	
-    va_start (args, msg);
-    vfprintf (stderr, msg, args);
-    va_end (args);
-}
-
 static svg_status_t
 _svg_parser_push_state (svg_parser_t		*parser,
 			const svg_parser_cb_t	*cb)
@@ -629,7 +401,7 @@ _svg_parser_new_leaf_element (svg_parser_t *parser,
 
 static svg_status_t
 _svg_parser_parse_anchor (svg_parser_t	*parser,
-			  const xmlChar	**attributes,
+			  const char	**attributes,
 			  svg_element_t	**group_element)
 {
     /* XXX: Currently ignoring all anchor elements */
@@ -638,7 +410,7 @@ _svg_parser_parse_anchor (svg_parser_t	*parser,
 
 static svg_status_t
 _svg_parser_parse_svg (svg_parser_t	*parser,
-		       const xmlChar	**attributes,
+		       const char	**attributes,
 		       svg_element_t	**group_element)
 {
     return _svg_parser_new_svg_group_element (parser, group_element);
@@ -646,7 +418,7 @@ _svg_parser_parse_svg (svg_parser_t	*parser,
 
 static svg_status_t
 _svg_parser_parse_group (svg_parser_t	*parser,
-			 const xmlChar	**attributes,
+			 const char	**attributes,
 			 svg_element_t	**group_element)
 {
     return _svg_parser_new_group_element (parser, group_element, SVG_ELEMENT_TYPE_GROUP);
@@ -654,7 +426,7 @@ _svg_parser_parse_group (svg_parser_t	*parser,
 
 static svg_status_t
 _svg_parser_parse_defs (svg_parser_t	*parser,
-			const xmlChar	**attributes,
+			const char	**attributes,
 			svg_element_t	**group_element)
 {
     return _svg_parser_new_group_element (parser, group_element, SVG_ELEMENT_TYPE_DEFS);
@@ -662,7 +434,7 @@ _svg_parser_parse_defs (svg_parser_t	*parser,
 
 static svg_status_t
 _svg_parser_parse_use (svg_parser_t	*parser,
-			const xmlChar	**attributes,
+			const char	**attributes,
 			svg_element_t	**group_element)
 {
     return _svg_parser_new_group_element (parser, group_element, SVG_ELEMENT_TYPE_USE);
@@ -670,7 +442,7 @@ _svg_parser_parse_use (svg_parser_t	*parser,
 
 static svg_status_t
 _svg_parser_parse_symbol (svg_parser_t	*parser,
-			  const xmlChar	**attributes,
+			  const char	**attributes,
 			  svg_element_t	**group_element)
 {
     return _svg_parser_new_group_element (parser, group_element, SVG_ELEMENT_TYPE_SYMBOL);
@@ -678,7 +450,7 @@ _svg_parser_parse_symbol (svg_parser_t	*parser,
 
 static svg_status_t
 _svg_parser_parse_path (svg_parser_t	*parser,
-			const xmlChar	**attributes,
+			const char	**attributes,
 			svg_element_t	**path_element)
 {
     return _svg_parser_new_leaf_element (parser,
@@ -688,7 +460,7 @@ _svg_parser_parse_path (svg_parser_t	*parser,
 
 static svg_status_t
 _svg_parser_parse_line (svg_parser_t	*parser,
-			const xmlChar	**attributes,
+			const char	**attributes,
 			svg_element_t	**path_element)
 {
     svg_status_t status;
@@ -708,7 +480,7 @@ _svg_parser_parse_line (svg_parser_t	*parser,
 
 static svg_status_t
 _svg_parser_parse_rect (svg_parser_t	*parser,
-			const xmlChar	**attributes,
+			const char	**attributes,
 			svg_element_t	**path_element)
 {
     svg_status_t status;
@@ -740,7 +512,7 @@ _svg_parser_parse_rect (svg_parser_t	*parser,
 
 static svg_status_t
 _svg_parser_parse_circle (svg_parser_t	*parser,
-			  const xmlChar	**attributes,
+			  const char	**attributes,
 			  svg_element_t	**path_element)
 {
     svg_status_t status;
@@ -761,7 +533,7 @@ _svg_parser_parse_circle (svg_parser_t	*parser,
 
 static svg_status_t
 _svg_parser_parse_ellipse (svg_parser_t		*parser,
-			   const xmlChar	**attributes,
+			   const char	**attributes,
 			   svg_element_t	**path_element)
 {
     svg_status_t status;
@@ -782,7 +554,7 @@ _svg_parser_parse_ellipse (svg_parser_t		*parser,
 
 static svg_status_t
 _svg_parser_parse_polygon (svg_parser_t		*parser,
-			   const xmlChar	**attributes,
+			   const char	**attributes,
 			   svg_element_t	**path_element)
 {
     svg_status_t status;
@@ -805,11 +577,11 @@ _svg_parser_parse_polygon (svg_parser_t		*parser,
 
 static svg_status_t
 _svg_parser_parse_polyline (svg_parser_t	*parser,
-			    const xmlChar	**attributes,
+			    const char	**attributes,
 			    svg_element_t	**path_element)
 {
     svg_status_t status;
-    const xmlChar *points;
+    const char *points;
     const char *p, *next;
     svg_path_t *path;
     double pt[2];
@@ -850,7 +622,7 @@ _svg_parser_parse_polyline (svg_parser_t	*parser,
 
 static svg_status_t
 _svg_parser_parse_text (svg_parser_t	*parser,
-			const xmlChar	**attributes,
+			const char	**attributes,
 			svg_element_t	**text_element)
 {
     svg_status_t status;
@@ -868,7 +640,7 @@ _svg_parser_parse_text (svg_parser_t	*parser,
 
 static svg_status_t
 _svg_parser_parse_image (svg_parser_t	*parser,
-			 const xmlChar	**attributes,
+			 const char	**attributes,
 			 svg_element_t	**image_element)
 {
     return _svg_parser_new_leaf_element (parser,
@@ -880,7 +652,7 @@ _svg_parser_parse_image (svg_parser_t	*parser,
 
 static svg_status_t
 _svg_parser_parse_linear_gradient (svg_parser_t	*parser,
-				   const xmlChar	**attributes,
+				   const char	**attributes,
 				   svg_element_t	**gradient_element)
 {
     svg_status_t status;
@@ -905,7 +677,7 @@ _svg_parser_parse_linear_gradient (svg_parser_t	*parser,
 
 static svg_status_t
 _svg_parser_parse_radial_gradient (svg_parser_t		*parser,
-				   const xmlChar	**attributes,
+				   const char	**attributes,
 				   svg_element_t	**gradient_element)
 {
     svg_status_t status;
@@ -931,7 +703,7 @@ _svg_parser_parse_radial_gradient (svg_parser_t		*parser,
    into an array when the gradient is done being parsed.  */
 static svg_status_t
 _svg_parser_parse_gradient_stop (svg_parser_t	*parser,
-				 const xmlChar	**attributes,
+				 const char	**attributes,
 				 svg_element_t	**gradient_element)
 {
     svg_style_t style;
@@ -939,7 +711,7 @@ _svg_parser_parse_gradient_stop (svg_parser_t	*parser,
     double offset;
     double opacity;
     svg_color_t color;
-    const xmlChar* color_str;
+    const char* color_str;
     svg_element_t *group_element;
 
     gradient_element = NULL;
@@ -953,7 +725,7 @@ _svg_parser_parse_gradient_stop (svg_parser_t	*parser,
 
     /* XXX: This ad-hoc style parsing breaks inheritance I believe. */
     _svg_style_init_empty (&style, parser->svg);
-    style.flags = 0;
+    style.flags = SVG_STYLE_FLAG_NONE;
     _svg_style_apply_attributes (&style, attributes);
     color = style.color;
     opacity = style.opacity;
@@ -977,7 +749,7 @@ _svg_parser_parse_gradient_stop (svg_parser_t	*parser,
 
 static svg_status_t
 _svg_parser_parse_pattern (svg_parser_t		*parser,
-			   const xmlChar	**attributes,
+			   const char	**attributes,
 			   svg_element_t	**pattern_element)
 {
     svg_status_t status;
@@ -998,7 +770,7 @@ _svg_parser_parse_pattern (svg_parser_t		*parser,
 
 static svg_status_t
 _svg_parser_parse_text_characters (svg_parser_t		*parser,
-				   const xmlChar	*ch,
+				   const char	*ch,
 				   int			len)
 {
     svg_status_t status;
