@@ -349,10 +349,10 @@ _svg_path_render (svg_path_t		*path,
 
 svg_status_t
 _svg_path_apply_attributes (svg_path_t		*path,
-			    const xmlChar	**attributes)
+			    const char		**attributes)
 {
     svg_status_t status;
-    const xmlChar *path_str;
+    const char *path_str;
 
     if (_svg_path_is_empty (path)) {
 	_svg_attribute_get_string (attributes, "d", &path_str, NULL);
@@ -384,9 +384,9 @@ _svg_path_cmd_info_lookup (char cmd_char, const svg_path_cmd_info_t **cmd_info)
 }
 
 svg_status_t
-_svg_path_add_from_str (svg_path_t *path, const xmlChar *path_str)
+_svg_path_add_from_str (svg_path_t *path, const char *path_str)
 {
-    const xmlChar *s;
+    const char *s;
     const char *end;
     svg_status_t status;
     const svg_path_cmd_info_t *cmd_info;
