@@ -26,86 +26,86 @@
 #include "svgint.h"
 
 static svg_status_t
-_svg_style_parse_color (svg_style_t *style, const xmlChar *str);
+_svg_style_parse_color (svg_style_t *style, const char *str);
 
 static svg_status_t
-_svg_style_parse_display (svg_style_t *style, const xmlChar *str);
+_svg_style_parse_display (svg_style_t *style, const char *str);
 
 static svg_status_t
-_svg_style_parse_fill_opacity (svg_style_t *style, const xmlChar *str);
+_svg_style_parse_fill_opacity (svg_style_t *style, const char *str);
 
 static svg_status_t
-_svg_style_parse_fill_paint (svg_style_t *style, const xmlChar *str);
+_svg_style_parse_fill_paint (svg_style_t *style, const char *str);
 
 static svg_status_t
-_svg_style_parse_fill_rule (svg_style_t *style, const xmlChar *str);
+_svg_style_parse_fill_rule (svg_style_t *style, const char *str);
 
 static svg_status_t
-_svg_style_parse_font_family (svg_style_t *style, const xmlChar *str);
+_svg_style_parse_font_family (svg_style_t *style, const char *str);
 
 static svg_status_t
-_svg_style_parse_font_size (svg_style_t *style, const xmlChar *str);
+_svg_style_parse_font_size (svg_style_t *style, const char *str);
 
 static svg_status_t
-_svg_style_parse_font_style (svg_style_t *style, const xmlChar *str);
+_svg_style_parse_font_style (svg_style_t *style, const char *str);
 
 static svg_status_t
-_svg_style_parse_font_weight (svg_style_t *style, const xmlChar *str);
+_svg_style_parse_font_weight (svg_style_t *style, const char *str);
 
 static svg_status_t
-_svg_style_parse_opacity (svg_style_t *style, const xmlChar *str);
+_svg_style_parse_opacity (svg_style_t *style, const char *str);
 
 static svg_status_t
-_svg_style_parse_stroke_dash_array (svg_style_t *style, const xmlChar *str);
+_svg_style_parse_stroke_dash_array (svg_style_t *style, const char *str);
 
 static svg_status_t
-_svg_style_parse_stroke_dash_offset (svg_style_t *style, const xmlChar *str);
+_svg_style_parse_stroke_dash_offset (svg_style_t *style, const char *str);
 
 static svg_status_t
-_svg_style_parse_stroke_line_cap (svg_style_t *style, const xmlChar *str);
+_svg_style_parse_stroke_line_cap (svg_style_t *style, const char *str);
 
 static svg_status_t
-_svg_style_parse_stroke_line_join (svg_style_t *style, const xmlChar *str);
+_svg_style_parse_stroke_line_join (svg_style_t *style, const char *str);
 
 static svg_status_t
-_svg_style_parse_stroke_miter_limit (svg_style_t *style, const xmlChar *str);
+_svg_style_parse_stroke_miter_limit (svg_style_t *style, const char *str);
 
 static svg_status_t
-_svg_style_parse_stroke_opacity (svg_style_t *style, const xmlChar *str);
+_svg_style_parse_stroke_opacity (svg_style_t *style, const char *str);
 
 static svg_status_t
-_svg_style_parse_stroke_paint (svg_style_t *style, const xmlChar *str);
+_svg_style_parse_stroke_paint (svg_style_t *style, const char *str);
 
 static svg_status_t
-_svg_style_parse_stroke_width (svg_style_t *style, const xmlChar *str);
+_svg_style_parse_stroke_width (svg_style_t *style, const char *str);
 
 static svg_status_t
-_svg_style_parse_text_anchor (svg_style_t *style, const xmlChar *str);
+_svg_style_parse_text_anchor (svg_style_t *style, const char *str);
 
 static svg_status_t
-_svg_style_parse_visibility (svg_style_t *style, const xmlChar *str);
+_svg_style_parse_visibility (svg_style_t *style, const char *str);
 
 static svg_status_t
-_svg_style_parse_stop_color (svg_style_t *style, const xmlChar *str);
+_svg_style_parse_stop_color (svg_style_t *style, const char *str);
 
 static svg_status_t
-_svg_style_parse_stop_opacity (svg_style_t *style, const xmlChar *str);
+_svg_style_parse_stop_opacity (svg_style_t *style, const char *str);
 
 static svg_status_t
 _svg_style_str_to_opacity (const char *str, double *ret);
 
 static svg_status_t
 _svg_style_parse_nv_pair (svg_style_t	*style,
-			  const xmlChar	*nv_pair);
+			  const char	*nv_pair);
 
 static svg_status_t
 _svg_style_parse_style_str (svg_style_t		*style,
-			    const xmlChar	*str);
+			    const char	*str);
 
 typedef struct svg_style_parse_map {
-    const xmlChar	*name;
-    svg_status_t 	(*parse) (svg_style_t *style, const xmlChar *value);
-    const xmlChar	*default_value;
+    const char	*name;
+    svg_status_t 	(*parse) (svg_style_t *style, const char *value);
+    const char		*default_value;
 } svg_style_parse_map_t;
 
 static const svg_style_parse_map_t SVG_STYLE_PARSE_MAP[] = {
@@ -166,9 +166,17 @@ _svg_style_init_empty (svg_style_t *style, svg_t *svg)
     style->flags = SVG_STYLE_FLAG_NONE;
     style->font_family = NULL;
     _svg_length_init_from_str (&style->font_size, "10px");
+    style->num_dashes = 0;
     style->stroke_dash_array = NULL;
     style->stroke_dash_offset.value = 0;
 
+    /* initialize unused elements so copies are predictable */
+    style->stroke_line_cap = SVG_STROKE_LINE_CAP_BUTT;
+    style->stroke_line_join = SVG_STROKE_LINE_JOIN_MITER;
+    style->stroke_miter_limit = 4.0;
+    style->stroke_opacity = 1.0;
+    style->fill_opacity = 1.0;
+    
     /* opacity is not inherited */
     style->flags |= SVG_STYLE_FLAG_OPACITY;
     style->opacity = 1.0;
@@ -182,7 +190,49 @@ _svg_style_init_empty (svg_style_t *style, svg_t *svg)
 svg_status_t
 _svg_style_init_copy (svg_style_t *style, svg_style_t *other)
 {
-	*style = *other;
+    style->svg = other->svg;
+
+    style->flags = other->flags;
+
+    style->fill_opacity = other->fill_opacity;
+    style->fill_paint = other->fill_paint;
+    style->fill_rule = other->fill_rule;
+
+    if (other->font_family) {
+	style->font_family = strdup (other->font_family);
+	if (style->font_family == NULL)
+	    return SVG_STATUS_NO_MEMORY;
+    } else {
+	style->font_family = NULL;
+    }
+
+    style->font_size = other->font_size;
+    style->font_style = other->font_style;
+    style->font_weight = other->font_weight;
+
+    style->opacity = other->opacity;
+
+    style->num_dashes = other->num_dashes;
+    if (style->num_dashes) {
+	style->stroke_dash_array = malloc (style->num_dashes * sizeof (double));
+	if (style->stroke_dash_array == NULL)
+	    return SVG_STATUS_NO_MEMORY;
+	memcpy (style->stroke_dash_array, other->stroke_dash_array,
+		style->num_dashes * sizeof (double));
+    } else {
+	style->stroke_dash_array = NULL;
+    }
+    style->stroke_dash_offset = other->stroke_dash_offset;
+
+    style->stroke_line_cap = other->stroke_line_cap;
+    style->stroke_line_join = other->stroke_line_join;
+    style->stroke_miter_limit = other->stroke_miter_limit;
+    style->stroke_opacity = other->stroke_opacity;
+    style->stroke_paint = other->stroke_paint;
+    style->stroke_width = other->stroke_width;
+
+    style->color = other->color;
+    style->text_anchor = other->text_anchor;
 
     return SVG_STATUS_SUCCESS;
 }
@@ -216,6 +266,11 @@ _svg_style_deinit (svg_style_t *style)
 	free (style->font_family);
     style->font_family = NULL;
 
+    if (style->stroke_dash_array)
+	free (style->stroke_dash_array);
+    style->stroke_dash_array = NULL;
+    style->num_dashes = 0;
+    
     style->flags = SVG_STYLE_FLAG_NONE;
 
     return SVG_STATUS_SUCCESS;
@@ -241,7 +296,7 @@ _svg_style_str_to_opacity (const char *str, double *ret)
 }
 
 static svg_status_t
-_svg_style_parse_color (svg_style_t *style, const xmlChar *str)
+_svg_style_parse_color (svg_style_t *style, const char *str)
 {
     svg_status_t status;
 
@@ -258,7 +313,7 @@ _svg_style_parse_color (svg_style_t *style, const xmlChar *str)
 }
 
 static svg_status_t
-_svg_style_parse_fill_opacity (svg_style_t *style, const xmlChar *str)
+_svg_style_parse_fill_opacity (svg_style_t *style, const char *str)
 {
     svg_status_t status;
 
@@ -272,7 +327,7 @@ _svg_style_parse_fill_opacity (svg_style_t *style, const xmlChar *str)
 }
 
 static svg_status_t
-_svg_style_parse_fill_paint (svg_style_t *style, const xmlChar *str)
+_svg_style_parse_fill_paint (svg_style_t *style, const char *str)
 {
     svg_status_t status;
 
@@ -286,7 +341,7 @@ _svg_style_parse_fill_paint (svg_style_t *style, const xmlChar *str)
 }
 
 static svg_status_t
-_svg_style_parse_fill_rule (svg_style_t *style, const xmlChar *str)
+_svg_style_parse_fill_rule (svg_style_t *style, const char *str)
 {
     if (strcmp (str, "evenodd") == 0)
 	style->fill_rule = SVG_FILL_RULE_EVEN_ODD;
@@ -302,7 +357,7 @@ _svg_style_parse_fill_rule (svg_style_t *style, const xmlChar *str)
 }
 
 static svg_status_t
-_svg_style_parse_font_family (svg_style_t *style, const xmlChar *str)
+_svg_style_parse_font_family (svg_style_t *style, const char *str)
 {
     free (style->font_family);
     style->font_family = strdup (str);
@@ -315,7 +370,7 @@ _svg_style_parse_font_family (svg_style_t *style, const xmlChar *str)
 }
 
 static svg_status_t
-_svg_style_parse_font_size (svg_style_t *style, const xmlChar *str)
+_svg_style_parse_font_size (svg_style_t *style, const char *str)
 {
     svg_status_t status;
 
@@ -329,7 +384,7 @@ _svg_style_parse_font_size (svg_style_t *style, const xmlChar *str)
 }
 
 static svg_status_t
-_svg_style_parse_font_style (svg_style_t *style, const xmlChar *str)
+_svg_style_parse_font_style (svg_style_t *style, const char *str)
 {
     if (strcmp (str, "normal") == 0)
 	style->font_style = SVG_FONT_STYLE_NORMAL;
@@ -346,7 +401,7 @@ _svg_style_parse_font_style (svg_style_t *style, const xmlChar *str)
 }
 
 static svg_status_t
-_svg_style_parse_font_weight (svg_style_t *style, const xmlChar *str)
+_svg_style_parse_font_weight (svg_style_t *style, const char *str)
 {
     if (strcmp (str, "normal") == 0)
 	style->font_weight = 400;
@@ -370,7 +425,7 @@ _svg_style_parse_font_weight (svg_style_t *style, const xmlChar *str)
 }
 
 static svg_status_t
-_svg_style_parse_opacity (svg_style_t *style, const xmlChar *str)
+_svg_style_parse_opacity (svg_style_t *style, const char *str)
 {
     svg_status_t status;
 
@@ -384,7 +439,7 @@ _svg_style_parse_opacity (svg_style_t *style, const xmlChar *str)
 }
 
 static svg_status_t
-_svg_style_parse_stroke_dash_array (svg_style_t *style, const xmlChar *str)
+_svg_style_parse_stroke_dash_array (svg_style_t *style, const char *str)
 {
     svgint_status_t status;
     double *new_dash_array;
@@ -421,7 +476,7 @@ _svg_style_parse_stroke_dash_array (svg_style_t *style, const xmlChar *str)
 }
 
 static svg_status_t
-_svg_style_parse_stroke_dash_offset (svg_style_t *style, const xmlChar *str)
+_svg_style_parse_stroke_dash_offset (svg_style_t *style, const char *str)
 {
     svg_status_t status;
 
@@ -435,7 +490,7 @@ _svg_style_parse_stroke_dash_offset (svg_style_t *style, const xmlChar *str)
 }
 
 static svg_status_t
-_svg_style_parse_stroke_line_cap (svg_style_t *style, const xmlChar *str)
+_svg_style_parse_stroke_line_cap (svg_style_t *style, const char *str)
 {
     if (strcmp (str, "butt") == 0)
 	style->stroke_line_cap = SVG_STROKE_LINE_CAP_BUTT;
@@ -453,7 +508,7 @@ _svg_style_parse_stroke_line_cap (svg_style_t *style, const xmlChar *str)
 }
 
 static svg_status_t
-_svg_style_parse_stroke_line_join (svg_style_t *style, const xmlChar *str)
+_svg_style_parse_stroke_line_join (svg_style_t *style, const char *str)
 {
     if (strcmp (str, "miter") == 0)
 	style->stroke_line_join = SVG_STROKE_LINE_JOIN_MITER;
@@ -471,7 +526,7 @@ _svg_style_parse_stroke_line_join (svg_style_t *style, const xmlChar *str)
 }
 
 static svg_status_t
-_svg_style_parse_stroke_miter_limit (svg_style_t *style, const xmlChar *str)
+_svg_style_parse_stroke_miter_limit (svg_style_t *style, const char *str)
 {
     const char *end;
 
@@ -485,7 +540,7 @@ _svg_style_parse_stroke_miter_limit (svg_style_t *style, const xmlChar *str)
 }
 
 static svg_status_t
-_svg_style_parse_stroke_opacity (svg_style_t *style, const xmlChar *str)
+_svg_style_parse_stroke_opacity (svg_style_t *style, const char *str)
 {
     svg_status_t status;
 
@@ -499,7 +554,7 @@ _svg_style_parse_stroke_opacity (svg_style_t *style, const xmlChar *str)
 }
 
 static svg_status_t
-_svg_style_parse_stroke_paint (svg_style_t *style, const xmlChar *str)
+_svg_style_parse_stroke_paint (svg_style_t *style, const char *str)
 {
     svg_status_t status;
 
@@ -513,7 +568,7 @@ _svg_style_parse_stroke_paint (svg_style_t *style, const xmlChar *str)
 }
 
 static svg_status_t
-_svg_style_parse_stroke_width (svg_style_t *style, const xmlChar *str)
+_svg_style_parse_stroke_width (svg_style_t *style, const char *str)
 {
     svg_status_t status;
 
@@ -527,7 +582,7 @@ _svg_style_parse_stroke_width (svg_style_t *style, const xmlChar *str)
 }
 
 static svg_status_t
-_svg_style_parse_text_anchor (svg_style_t *style, const xmlChar *str)
+_svg_style_parse_text_anchor (svg_style_t *style, const char *str)
 {
     if (strcmp (str, "start") == 0)
 	style->text_anchor = SVG_TEXT_ANCHOR_START;
@@ -544,7 +599,7 @@ _svg_style_parse_text_anchor (svg_style_t *style, const xmlChar *str)
 }
 
 static svg_status_t
-_svg_style_parse_visibility (svg_style_t *style, const xmlChar *str)
+_svg_style_parse_visibility (svg_style_t *style, const char *str)
 {
     /* XXX: Do we care about the CSS2 definitions for these? */
     if (strcmp (str, "hidden") == 0 || strcmp (str, "collapse") == 0)
@@ -558,7 +613,7 @@ _svg_style_parse_visibility (svg_style_t *style, const xmlChar *str)
 }
 
 static svg_status_t
-_svg_style_parse_display (svg_style_t *style, const xmlChar *str)
+_svg_style_parse_display (svg_style_t *style, const char *str)
 {
     /* XXX: Do we care about the CSS2 definitions for these? */
     if (strcmp (str, "none") == 0)
@@ -579,7 +634,7 @@ _svg_style_parse_display (svg_style_t *style, const xmlChar *str)
 }
 
 static svg_status_t
-_svg_style_parse_stop_color (svg_style_t *style, const xmlChar *str)
+_svg_style_parse_stop_color (svg_style_t *style, const char *str)
 {
     svg_status_t status;
 
@@ -594,7 +649,7 @@ _svg_style_parse_stop_color (svg_style_t *style, const xmlChar *str)
 
 
 static svg_status_t
-_svg_style_parse_stop_opacity (svg_style_t *style, const xmlChar *str)
+_svg_style_parse_stop_opacity (svg_style_t *style, const char *str)
 {
     svg_status_t status;
     double opacity = 1.0;
@@ -611,9 +666,9 @@ _svg_style_parse_stop_opacity (svg_style_t *style, const xmlChar *str)
 
 
 static svg_status_t
-_svg_style_split_nv_pair_alloc (const xmlChar	*nv_pair,
-				xmlChar		**name,
-				xmlChar		**value)
+_svg_style_split_nv_pair_alloc (const char	*nv_pair,
+				char		**name,
+				char		**value)
 {
     char *colon;
     const char *v_start;
@@ -646,10 +701,10 @@ _svg_style_split_nv_pair_alloc (const xmlChar	*nv_pair,
 /* Parse a CSS2 style argument */
 static svg_status_t
 _svg_style_parse_nv_pair (svg_style_t	*style,
-			  const xmlChar	*nv_pair)
+			  const char	*nv_pair)
 {
     unsigned int i;
-    xmlChar *name, *value;
+    char *name, *value;
     svg_status_t status;
 
     status = _svg_style_split_nv_pair_alloc (nv_pair, &name, &value);
@@ -688,7 +743,7 @@ _svg_style_parse_nv_pair (svg_style_t	*style,
 */
 static svg_status_t
 _svg_style_parse_style_str (svg_style_t		*style,
-			    const xmlChar	*str)
+			    const char	*str)
 {
     int start, end;
     char *nv_pair;
@@ -713,11 +768,11 @@ _svg_style_parse_style_str (svg_style_t		*style,
 
 svg_status_t
 _svg_style_apply_attributes (svg_style_t	*style, 
-			     const xmlChar	**attributes)
+			     const char		**attributes)
 {
     unsigned int i;
     svg_status_t status;
-    const xmlChar *style_str, *str;
+    const char *style_str, *str;
 
     _svg_attribute_get_string (attributes, "style", &style_str, NULL);
 
