@@ -23,10 +23,30 @@
 #ifndef SVGINT_H
 #define SVGINT_H
 
+#ifdef LIBSVG_EXPAT
+
+#include <expat.h>
+#include "svg_hash.h"
+
+typedef XML_Char xmlChar;
+typedef XML_Parser svg_xml_parser_context_t;
+
+#else
+
 #include <libxml/SAX.h>
 #include <libxml/xmlmemory.h>
 #include <libxml/hash.h>
 
+typedef xmlParserCtxtPtr svg_xml_parser_context_t;
+typedef xmlHashTable svg_xml_hash_table_t;
+
+#define _svg_xml_hash_add_entry		xmlHashAddEntry
+#define _svg_xml_hash_lookup		xmlHashLookup
+#define _svg_xml_hash_create		xmlHashCreate
+#define _svg_xml_hash_free(_table)	xmlHashFree(_table, NULL)
+
+#endif
+
 #include "config.h"
 #include "svg_version.h"
 #include "svg.h"
@@ -217,7 +237,7 @@ struct svg_group {
 typedef struct svg_text {
     svg_length_t x;
     svg_length_t y;
-    unsigned char *chars;
+    char *chars;
     unsigned int len;
 } svg_text_t;
 
@@ -287,7 +307,7 @@ struct svg_element {
 
     svg_element_type_t type;
 
-    xmlChar *id;
+    char *id;
 
     union {
 	svg_group_t group;
@@ -306,11 +326,11 @@ struct svg_element {
 typedef struct svg_parser svg_parser_t;
 
 typedef svg_status_t (svg_parser_parse_element_t)(svg_parser_t	*parser,
-						  const xmlChar	**attributes,
+						  const char	**attributes,
 						  svg_element_t	**element_ret);
 
 typedef svg_status_t (svg_parser_parse_characters_t) (svg_parser_t	*parser,
-						      const xmlChar	*ch,
+						      const char	*ch,
 						      int		len);
 
 typedef struct svg_parser_cb {
@@ -330,12 +350,12 @@ struct svg_parser {
 
     svg_t *svg;
 
-    xmlParserCtxtPtr ctxt;
+    svg_xml_parser_context_t ctxt;
 
     unsigned int unknown_element_depth;
     svg_parser_state_t *state;
 
-    xmlHashTable *entities;
+    svg_xml_hash_table_t *entities;
 
     svg_status_t status;
 };
@@ -347,7 +367,7 @@ struct svg {
 
     svg_element_t *group_element;
 
-    xmlHashTable *element_ids;
+    svg_xml_hash_table_t *element_ids;
 
     svg_parser_t parser;
 
@@ -362,7 +382,7 @@ svg_status_t
 _svg_store_element_by_id (svg_t *svg, svg_element_t *element);
 
 svg_status_t
-_svg_fetch_element_by_id (svg_t *svg, const xmlChar *id, svg_element_t **element_ret);
+_svg_fetch_element_by_id (svg_t *svg, const char *id, svg_element_t **element_ret);
 
 /* libsvg_features.c */
 
@@ -375,22 +395,22 @@ void libsvg_postinit(void *app, void *modinfo);
 /* svg_attribute.c */
 
 svgint_status_t
-_svg_attribute_get_double (const xmlChar	**attributes,
-			   const xmlChar	*name,
-			   double		*value,
-			   double		default_value);
+_svg_attribute_get_double (const char	**attributes,
+			   const char	*name,
+			   double	*value,
+			   double	 default_value);
 
 svgint_status_t
-_svg_attribute_get_string (const xmlChar	**attributes,
-			   const xmlChar	*name,
-			   const xmlChar	**value,
-			   const xmlChar	*default_value);
+_svg_attribute_get_string (const char	**attributes,
+			   const char	*name,
+			   const char	**value,
+			   const char	*default_value);
 
 svgint_status_t
-_svg_attribute_get_length (const xmlChar	**attributes,
-			   const xmlChar	*name,
-			   svg_length_t		*value,
-			   const xmlChar	*default_value);
+_svg_attribute_get_length (const char	**attributes,
+			   const char	*name,
+			   svg_length_t	*value,
+			   const char	*default_value);
 
 /* svg_color.c */
 
@@ -436,7 +456,7 @@ _svg_element_clone (svg_element_t	**element,
 
 svg_status_t
 _svg_element_apply_attributes (svg_element_t	*group_element,
-			       const xmlChar	**attributes);
+			       const char	**attributes);
 
 svg_status_t
 _svg_element_parse_aspect_ratio (const char *aspect_ratio_str,
@@ -477,7 +497,7 @@ _svg_gradient_add_stop (svg_gradient_t *gradient,
 svg_status_t
 _svg_gradient_apply_attributes (svg_gradient_t	*gradient,
 				svg_t		*svg,
-				const xmlChar	**attributes);
+				const char	**attributes);
 
 /* svg_group.c */
 
@@ -506,15 +526,15 @@ _svg_symbol_render (svg_element_t	*group,
 
 svg_status_t
 _svg_group_apply_svg_attributes (svg_group_t	*group,
-				 const xmlChar	**attributes);
+				 const char	**attributes);
 
 svg_status_t
 _svg_group_apply_group_attributes (svg_group_t		*group,
-				   const xmlChar	**attributes);
+				   const char		**attributes);
 
 svg_status_t
 _svg_group_apply_use_attributes (svg_element_t		*group,
-				 const xmlChar		**attributes);
+				 const char		**attributes);
 
 svg_status_t
 _svg_group_get_size (svg_group_t *group, svg_length_t *width, svg_length_t *height);
@@ -533,7 +553,7 @@ _svg_image_deinit (svg_image_t *image);
 
 svg_status_t
 _svg_image_apply_attributes (svg_image_t	*image,
-			     const xmlChar	**attributes);
+			     const char		**attributes);
 
 svg_status_t
 _svg_image_render (svg_image_t		*image,
@@ -560,7 +580,7 @@ _svg_length_deinit (svg_length_t *length);
 /* svg_paint.c */
 
 svg_status_t
-_svg_paint_init (svg_paint_t *paint, svg_t *svg, const xmlChar *str);
+_svg_paint_init (svg_paint_t *paint, svg_t *svg, const char *str);
 
 svg_status_t
 _svg_paint_deinit (svg_paint_t *paint);
@@ -582,6 +602,20 @@ _svg_parser_parse_chunk (svg_parser_t *parser, const char *buf, size_t count);
 svg_status_t
 _svg_parser_end (svg_parser_t *parser);
 
+void
+_svg_parser_sax_start_element (void		*closure,
+			       const xmlChar	*name,
+			       const xmlChar	**atts);
+
+void
+_svg_parser_sax_end_element (void		*closure,
+			     const xmlChar	*name);
+
+void
+_svg_parser_sax_characters (void		*closure,
+			    const xmlChar	*ch,
+			    int			len);
+
 /* svg_path.c */
 
 svg_status_t
@@ -646,10 +680,10 @@ _svg_rect_render (svg_rect_element_t	*rect,
 
 svg_status_t
 _svg_path_apply_attributes (svg_path_t		*path,
-			    const xmlChar	**attributes);
+			    const char		**attributes);
 
 svg_status_t
-_svg_path_add_from_str (svg_path_t *path, const xmlChar *path_str);
+_svg_path_add_from_str (svg_path_t *path, const char *path_str);
 
 svg_status_t
 _svg_path_move_to (svg_path_t *path, double x, double y);
@@ -754,7 +788,7 @@ _svg_pattern_deinit (svg_pattern_t *pattern);
 
 svg_status_t
 _svg_pattern_apply_attributes (svg_pattern_t	*pattern,
-			       const xmlChar	**attributes);
+			       const char	**attributes);
 
 svg_status_t
 _svg_pattern_render (svg_element_t		*pattern,
@@ -799,7 +833,7 @@ _svg_style_render (svg_style_t		*style,
 
 svg_status_t
 _svg_style_apply_attributes (svg_style_t	*style, 
-			     const xmlChar	**attributes);
+			     const char		**attributes);
 
 double
 _svg_style_get_opacity (svg_style_t *style);
@@ -824,7 +858,7 @@ _svg_text_deinit (svg_text_t *text);
 
 svg_status_t
 _svg_text_append_chars (svg_text_t	*text,
-			const xmlChar	*chars,
+			const char	*chars,
 			int		len);
 
 svg_status_t
@@ -834,7 +868,7 @@ _svg_text_render (svg_text_t		*text,
 
 svg_status_t
 _svg_text_apply_attributes (svg_text_t		*text,
-			    const xmlChar	**attributes);
+			    const char		**attributes);
 
 /* svg_transform.c */
 
@@ -905,11 +939,6 @@ _svg_transform_multiply_into_left (svg_transform_t *t1, const svg_transform_t *t
 svg_status_t
 _svg_transform_multiply_into_right (const svg_transform_t *t1, svg_transform_t *t2);
 
-svg_status_t
-_svg_transform_multiply (const svg_transform_t	*t1,
-			 const svg_transform_t	*t2,
-			 svg_transform_t	*new);
-
 svg_status_t
 _svg_transform_render (svg_transform_t		*transform,
 		       svg_render_engine_t	*engine,
@@ -917,6 +946,6 @@ _svg_transform_render (svg_transform_t		*transform,
 
 svg_status_t
 _svg_transform_apply_attributes (svg_transform_t	*transform,
-				 const xmlChar		**attributes);
+				 const char		**attributes);
 
 #endif
